'use strict';

var _get = require('babel-runtime/helpers/get')['default'];

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _defineProperty = require('babel-runtime/helpers/define-property')['default'];

var _Object$assign = require('babel-runtime/core-js/object/assign')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _nedb = require('nedb');

var _nedb2 = _interopRequireDefault(_nedb);

var _helpers = require('./helpers');

// By default, try to auto-compact the database every minute.
var defaultCompactionInterval = 60 * 1000;

/**
 * NeDB adapter.
 */

exports['default'] = function (Adapter) {
  return (function (_Adapter) {
    _inherits(NedbAdapter, _Adapter);

    function NedbAdapter() {
      _classCallCheck(this, NedbAdapter);

      _get(Object.getPrototypeOf(NedbAdapter.prototype), 'constructor', this).apply(this, arguments);
    }

    _createClass(NedbAdapter, [{
      key: 'connect',
      value: function connect() {
        var _this = this;

        var recordTypes = this.recordTypes;
        var options = this.options;
        var _options = this.options;
        var compactionInterval = _options.compactionInterval;
        var dbPath = _options.dbPath;

        delete options.filename;

        try {
          this.db = (0, _helpers.mapValues)(recordTypes, function (fields, type) {
            var db = new _nedb2['default'](_Object$assign({}, options, dbPath ? {
              filename: _path2['default'].join(dbPath, type + '.db')
            } : null));
            db.persistence.setAutocompactionInterval(compactionInterval ? compactionInterval : defaultCompactionInterval);
            return db;
          });
        } catch (error) {
          return _Promise.reject(error);
        }

        return _Promise.all(_Object$keys(this.db).map(function (type) {
          return new _Promise(function (resolve, reject) {
            return _this.db[type].loadDatabase(function (error) {
              return error ? reject(error) : resolve();
            });
          });
        })).then(function () {
          return null;
        });
      }
    }, {
      key: 'disconnect',
      value: function disconnect() {
        var _this2 = this;

        return _Promise.all(_Object$keys(this.db).map(function (key) {
          return new _Promise(function (resolve) {
            var db = _this2.db[key];

            // This auto compaction interval prevents the process from exiting.
            db.persistence.stopAutocompaction();

            // Internal hook to NeDB's executor which will run after all other
            // operations are done.
            db.executor.push({ fn: resolve, arguments: [] });
          });
        })).then(function () {
          return null;
        });
      }
    }, {
      key: 'find',
      value: function find(type, ids, options) {
        var _this3 = this;

        // Handle no-op.
        if (ids && !ids.length) return _get(Object.getPrototypeOf(NedbAdapter.prototype), 'find', this).call(this);
        if (!options) options = {};

        var query = {};

        try {
          if ('match' in options) _Object$assign(query, (0, _helpers.mapValues)(options.match, function (value) {
            return Array.isArray(value) ? { $in: value.map(_helpers.castValue) } : (0, _helpers.castValue)(value);
          }));

          if ('query' in options) _Object$assign(query, options.query);

          if (ids && ids.length) query[_helpers.idKey] = { $in: ids };
        } catch (error) {
          return _Promise.reject(error);
        }

        // Parallelize the find method with count method.
        return _Promise.all([new _Promise(function (resolve, reject) {
          var _db$type;

          var args = [query];

          if ('fields' in options) args.push((0, _helpers.mapValues)(options.fields, function (value) {
            return value ? 1 : 0;
          }));

          var find = (_db$type = _this3.db[type]).find.apply(_db$type, args);

          if ('sort' in options) find.sort((0, _helpers.mapValues)(options.sort, function (value) {
            return value ? 1 : -1;
          }));

          if ('offset' in options) find.skip(options.offset);

          if ('limit' in options) find.limit(options.limit);

          find.exec(function (error, records) {
            return error ? reject(error) : resolve(records.map(_helpers.outputRecord.bind(_this3, type)));
          });
        }), new _Promise(function (resolve, reject) {
          return _this3.db[type].count(query, function (error, count) {
            return error ? reject(error) : resolve(count);
          });
        })]).then(function (results) {
          // Set the count on the records array.
          results[0].count = results[1];
          return results[0];
        });
      }
    }, {
      key: 'create',
      value: function create(type, records) {
        var _this4 = this;

        var ConflictError = this.errors.ConflictError;

        return new _Promise(function (resolve, reject) {
          return _this4.db[type].insert(records.map(_helpers.inputRecord.bind(_this4, type)), function (error, result) {
            return error ? reject(error.errorType === 'uniqueViolated' ? new ConflictError('Duplicate key.') : error) : resolve(result.map(_helpers.outputRecord.bind(_this4, type)));
          });
        });
      }
    }, {
      key: 'update',
      value: function update(type, updates) {
        var _this5 = this;

        var keys = this.keys;

        return _Promise.all(updates.map(function (update) {
          return new _Promise(function (resolve, reject) {
            var modifiers = {};

            if ('replace' in update) modifiers.$set = update.replace;

            if ('push' in update) modifiers.$push = (0, _helpers.mapValues)(update.push, function (value) {
              return Array.isArray(value) ? { $each: value } : value;
            });

            if ('pull' in update) modifiers.$pull = (0, _helpers.mapValues)(update.pull, function (value) {
              return Array.isArray(value) ? { $in: value } : value;
            });

            // Custom update operators have precedence.
            _Object$assign(modifiers, update.operate);

            // Short circuit no-op.
            if (!_Object$keys(modifiers).length) return resolve(0);

            _this5.db[type].update(_defineProperty({}, _helpers.idKey, update[keys.primary]), modifiers, {}, function (error, number) {
              return error ? reject(error) : resolve(number);
            });
          });
        })).then(function (numbers) {
          return numbers.reduce(function (accumulator, number) {
            return accumulator + number;
          }, 0);
        });
      }
    }, {
      key: 'delete',
      value: function _delete(type, ids) {
        var _this6 = this;

        if (ids && !ids.length) return _get(Object.getPrototypeOf(NedbAdapter.prototype), 'delete', this).call(this);

        return new _Promise(function (resolve, reject) {
          return _this6.db[type].remove(ids && ids.length ? _defineProperty({}, _helpers.idKey, { $in: ids }) : {}, { multi: true }, function (error, number) {
            return error ? reject(error) : resolve(number);
          });
        });
      }
    }]);

    return NedbAdapter;
  })(Adapter);
};

module.exports = exports['default'];